#ifndef PLAYERS_COMMON_H
#define PLAYERS_COMMON_H

#include <stdio.h>
#include <gmodule.h>

struct playlist_item {
   char *name;
   char *title;
   char *artist;
   char *album;
   char *year;
   char *comment;
   char *path;
   int length;
   int pos;
   int stream_type;
   int delete_after;
   struct player_plugin *player;
   struct playlist_item *next;
   struct playlist_item *prev;
};

typedef struct {
   struct playlist_item *first;
   struct playlist_item *last;
   int length;
} PlayList;

#define TYPE_PLAYER   0
#define TYPE_ANALYSIS 1
#define TYPE_LOADER   2
#define TYPE_ALL      3

typedef struct _generic_plugin {
   char *id;
   char *version;
   char *copyright;
   GModule *handle;
   char *soname;
   int loaded;
   int type;
   void *data;
   struct _generic_plugin *next;
} GenericPlugin;

struct player_plugin {
   void (*init) (struct player_plugin * plug);
   void (*config) (char *left, char *right);
   void (*seek_to_frame) (int second);
   int (*play) (char *song);
   void (*pause) (void);
   void (*stop) (int playing);
   void (*get_info) (struct playlist_item * song);
   int streamable;
   GenericPlugin *parent;
};

#ifdef __cplusplus
extern "C" {
#endif 
   extern int setup_audio(int frequency, int stereo, int sign, int big, int sixteen);
   extern void set_bitrate(int bit);
   extern void close_audio(void);
   extern void flush_audio(void);
   extern int end_of_song;
   extern int parsebool(char *data);
   extern int audio_play(char *buffer, int size);
   extern int exists(char *s);

   extern void eaudio_register_ext(char *ext, struct player_plugin *plug);

   extern FILE *eaudio_fopen(char *path, char *mode);
   extern int eaudio_fclose(FILE * stream);
   extern size_t eaudio_fread(void *ptr, size_t size, size_t nmemb, FILE * stream);
   extern int eaudio_fseek(FILE * stream, long offset, int whence);
   extern int eaudio_getc(FILE * stream);
   extern int eaudio_feof(FILE * stream);
   extern long eaudio_ftell(FILE * stream);
   extern long eaudio_getsize(char *filename);

   extern void *__Emalloc(int size, const char *file, int line);
   extern void *__Erealloc(void *ptr, int size, const char *file, int line);
   extern void *__Ecalloc(int num, int size, const char *file, int line);
   extern void __Efree(void *ptr, const char *file, int line);
   extern char *duplicate(char *s);
   
#if defined(__FILE__) && defined(__LINE__)
#define Efree(x) __Efree(x, __FILE__, __LINE__)
#define Emalloc(x) __Emalloc(x, __FILE__, __LINE__)
#define Erealloc(x, y) __Erealloc(x, y, __FILE__, __LINE__)
#define Ecalloc(x, y) __Ecalloc(x, y, __FILE__, __LINE__)        
#else
#define Efree(x) free(x)
#define Emalloc(x) malloc(x)
#define Erealloc(x, y) realloc(x, y)
#define Ecalloc(x, y) calloc(x, y)
#endif

   /* stuff from ../eaudio/fileops.h -- if ya need more just add em here */
   extern char *homedir(int uid);
   extern char *username(int uid);
#ifdef __cplusplus
}

#endif

#endif
